// WeatherReport.cpp: implementation of the CWeatherReport class.
//
//////////////////////////////////////////////////////////////////////

#include <cstdio>
#include <iostream.h>
#include <iomanip.h>

#include "WeatherReport.h"
#include "Calendar.h"
#include "Climate.h"
#include "Dice.h"
#include "Stats.h"

CWeatherReport* CWeatherReport::m_pInstance = NULL;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CWeatherReport::CWeatherReport() : m_cWeather(0), m_cWind(0), m_cTemp(0), m_cPrecip(0), m_cOvercast(0), m_nDay(0), m_nMonth(0), m_nYear(0)
{
	m_pStat = new CStats();
	m_pCalendar = CCalendar::instance();
	m_pClimate = CClimate::instance();
	m_pYesterday = NULL;
	init();
}

CWeatherReport::~CWeatherReport()
{
	if (m_pStat != NULL)
	{
		delete m_pStat;
		m_pStat = NULL;
	}
	if (m_pYesterday != NULL)
	{
		delete m_pYesterday;
		m_pYesterday = NULL;
	}
	if (m_pInstance != NULL)
	{
		delete m_pInstance;
		m_pInstance = NULL;
	}
}

CWeatherReport& CWeatherReport::operator=(const CWeatherReport& rhs)
{
	m_cWeather = rhs.m_cWeather;
	m_cWind = rhs.m_cWind;
	m_cTemp = rhs.m_cTemp;
	m_cPrecip = rhs.m_cPrecip;
	m_cOvercast = rhs.m_cOvercast;
	m_pStat = NULL;
	m_pCalendar = NULL;
	m_pClimate = NULL;
	m_pYesterday = NULL;
	return *this;
}

CWeatherReport* CWeatherReport::instance()
{
	if (!m_pInstance)
	{
		m_pInstance = new CWeatherReport();
	}
	return m_pInstance;
}

void CWeatherReport::init()
{
    m_pStat->clear();
}

void CWeatherReport::generate(int nDay, int nMonth, int nYear)
{
	// set yesterday to today before creating a new today
	if (m_pYesterday == NULL)
	{
		m_pYesterday = new CWeatherReport();
	}
	*m_pYesterday = *this;

	m_nDay = nDay;
	m_nMonth = nMonth;
	m_nYear = nYear;

    // half the time, weather is the same as yesterday
    if (CDice::roll(100) > 50) 
	{
        m_cWeather.init();
	}
    else
    {
        m_cWeather.clearCatastrophe();
    }
    m_cWind.init();
    m_cTemp.init();
    m_cPrecip.init();
    m_cOvercast.init();

	// temp statistics
    m_pStat->updateTemp(nMonth, m_cTemp.getDegrees());

	// precip statistics(
	m_pStat->updatePrecip(m_cPrecip.getAmount(), m_cPrecip.getDuration(), m_cPrecip.isSnow());
    
    // count sunny days
    if (m_cWeather.getType() == CWeather::CLEAR) 
	{
        m_pStat->incrSunshine();
	}
}

void CWeatherReport::print() const
{
/* print today's weather */
    string precip;
	 float fSnowDepth = m_pStat->getSnowDepth();

	 cout << setiosflags( ios::fixed );
	 if ((1 == m_nDay) && (m_pCalendar->getFirstMonth() == m_nMonth))
	 {
		 printYearHeader();
	 }
	 if (1 == m_nDay)
	 {
		 printMonthHeader();
	 }
     cout << setw(4) << m_nDay;
	 cout << "       ";
	 cout << setiosflags(ios::left);
	 cout << setw(15) << m_cWeather.getName().c_str();
	 cout << setw(11) << m_cOvercast.getName().c_str();
	 cout << setw(7) << m_cTemp.getName().c_str();
	 cout << setw(1) << "(";
	 cout << setw(3) << m_cTemp.getDegrees();
	 cout << setw(3) << ")";
	 cout << setw(10) << m_cPrecip.getName().c_str();
	 cout << setw(10) << m_cWind.getName().c_str();
	 cout << resetiosflags(ios::left);
     if (fSnowDepth > 0.0) 
	 {
         cout << setw(5) << setprecision(1) << fSnowDepth << "\"";
	 }
     else
	 {
         cout << "      ";
	 }

	 cout << "  ";

	 bool bNoteWritten = false;
     if (m_cWeather.isCatastrophe())
	 {
         cout << "Catastrophe";
		 bNoteWritten = true;
	 }
	 if (m_cPrecip.getType() != CPrecip::NONE) 
     {
		 if (bNoteWritten) cout << ", ";
         if (m_cPrecip.isSnow()) 
             precip = (m_cPrecip.isStorm() ? "Blizzard" : "Snow");
         else
             precip = (m_cPrecip.isStorm() ? "Storm" : "Rain");
         cout << precip.c_str();
		 cout << " - " << setprecision(1) << m_cPrecip.getDuration() << " hours";
		 bNoteWritten = true;
     }
	 string strHoliday = m_pCalendar->getHoliday(m_nDay, m_nMonth);
	 if (strHoliday.length() != 0)
	 {
		 if (bNoteWritten) cout << ", ";
		 cout << strHoliday.c_str();
	 }
     cout << endl;

	 if (m_pCalendar->getMonthLength(m_nMonth) == m_nDay)
	 {
		 printMonthFooter();
	 }
	 if ((m_pCalendar->getMonthLength(m_nMonth) == m_nDay) && (m_pCalendar->getLastMonth() == m_nMonth))
	 {
		 printYearFooter();
	 }
}

void CWeatherReport::printYearHeader() const
{
	cout << setw(76) << "Snow" << endl;
	cout << "Year " << setw(4) << m_nYear;
	cout << "  " << setiosflags(ios::left) << setw(15) << "Weather" << setw(11) << "Sky" << setw(14) << "Temperature";
	cout << setw(10) << "Precip" << setw(11) << "Wind" << setw(7) << "Depth" << "Notes" << endl;
	cout << resetiosflags(ios::left);
}

void CWeatherReport::printYearFooter() const
{
	cout << "Sunshine = " << setw(3) << m_pStat->getSunshine() << " days" << endl;
	cout << "Rainfall = " << setw(4) << setprecision(1) << m_pStat->getRainfall() << " \"" << endl;
	cout << "Snowfall = " << setw(4) << setprecision(1) << m_pStat->getSnowfall() << " \"" << endl;
	cout << endl;
}

void CWeatherReport::printMonthHeader() const
{
	cout << setiosflags(ios::left);
	cout << setw(9) << m_pCalendar->getMonthName(m_nMonth).c_str() << endl;
	cout << resetiosflags(ios::left);
}

void CWeatherReport::printMonthFooter() const
{
	m_pStat->calculateAverage(m_nMonth);
	cout << "High Temp = " << setw(3) << m_pStat->getTempStat(m_nMonth, CStats::HIGH) << ", ";
	cout << "Low Temp = " << setw(3) << m_pStat->getTempStat(m_nMonth, CStats::LOW) << ", ";
	cout << "Average Temp = " << setw(3) << m_pStat->getTempStat(m_nMonth, CStats::AVERAGE) << endl;
	cout << endl;
}

bool CWeatherReport::isSnow() const
{
	return m_cPrecip.isSnow();
}

int CWeatherReport::getWeather() const
{
	return m_cWeather.getType();
}

int CWeatherReport::getWind() const
{
	return m_cWind.getType();
}

int CWeatherReport::getTemp() const
{
	return m_cTemp.getType();
}

int CWeatherReport::getTempDegrees() const
{
	return m_cTemp.getDegrees();
}

int CWeatherReport::getPrecip() const
{
	return m_cPrecip.getType();
}

int CWeatherReport::getOvercast() const
{
	return m_cOvercast.getType();
}

int CWeatherReport::getMonth() const
{
	return m_nMonth;
}

CClimate* CWeatherReport::getClimate() const
{
	return m_pClimate;
}

CCalendar* CWeatherReport::getCalendar() const
{
	return m_pCalendar;
}

CWeatherReport* CWeatherReport::getYesterday() const
{
	return m_pYesterday;
}

void CWeatherReport::setWeather(int nWeather)
{
	m_cWeather.setType(nWeather);
}

/* initialize last year's winter */
void CWeatherReport::initWinter()
{
     /* calculate snow depth for Oct. thru Dec. */
	CCalendar* pCalendar = CCalendar::instance();
	for (int month = pCalendar->getLastMonth()-2; month <= pCalendar->getLastMonth(); month++)
	{
        for (int day = 1; day <= pCalendar->getMonthLength(month); day++)
        {
            generate(day, month, m_nYear-1);
        }
	}
}

